import { Formatter } from 'picocolors/types';
import { TransformResult } from 'vite';
import { OptionsReceived } from 'pretty-format';
import { MessagePort } from 'worker_threads';
export { assert, default as chai, expect, should } from 'chai';
import { spyOn, spy } from 'tinyspy';
export { spy, spyOn } from 'tinyspy';

declare const EXPECTED_COLOR: Formatter;
declare const RECEIVED_COLOR: Formatter;
declare const INVERTED_COLOR: Formatter;
declare const BOLD_WEIGHT: Formatter;
declare const DIM_COLOR: Formatter;
declare type MatcherHintOptions = {
    comment?: string;
    expectedColor?: Formatter;
    isDirectExpectCall?: boolean;
    isNot?: boolean;
    promise?: string;
    receivedColor?: Formatter;
    secondArgument?: string;
    secondArgumentColor?: Formatter;
};
declare function matcherHint(matcherName: string, received?: string, expected?: string, options?: MatcherHintOptions): string;
declare const stringify: (object: unknown, maxDepth?: number) => string;
declare const printReceived: (object: unknown) => string;
declare const printExpected: (value: unknown) => string;
declare type DiffOptions = {
    aAnnotation?: string;
    aColor?: Formatter;
    aIndicator?: string;
    bAnnotation?: string;
    bColor?: Formatter;
    bIndicator?: string;
    changeColor?: Formatter;
    changeLineTrailingSpaceColor?: Formatter;
    commonColor?: Formatter;
    commonIndicator?: string;
    commonLineTrailingSpaceColor?: Formatter;
    contextLines?: number;
    emptyFirstOrLastLinePlaceholder?: string;
    expand?: boolean;
    includeChangeCounts?: boolean;
    omitAnnotationLines?: boolean;
    patchColor?: Formatter;
    compareKeys?: any;
};
declare function diff(a: any, b: any, options?: DiffOptions): string;

declare const jestMatcherUtils_EXPECTED_COLOR: typeof EXPECTED_COLOR;
declare const jestMatcherUtils_RECEIVED_COLOR: typeof RECEIVED_COLOR;
declare const jestMatcherUtils_INVERTED_COLOR: typeof INVERTED_COLOR;
declare const jestMatcherUtils_BOLD_WEIGHT: typeof BOLD_WEIGHT;
declare const jestMatcherUtils_DIM_COLOR: typeof DIM_COLOR;
type jestMatcherUtils_MatcherHintOptions = MatcherHintOptions;
declare const jestMatcherUtils_matcherHint: typeof matcherHint;
declare const jestMatcherUtils_stringify: typeof stringify;
declare const jestMatcherUtils_printReceived: typeof printReceived;
declare const jestMatcherUtils_printExpected: typeof printExpected;
type jestMatcherUtils_DiffOptions = DiffOptions;
declare const jestMatcherUtils_diff: typeof diff;
declare namespace jestMatcherUtils {
  export {
    jestMatcherUtils_EXPECTED_COLOR as EXPECTED_COLOR,
    jestMatcherUtils_RECEIVED_COLOR as RECEIVED_COLOR,
    jestMatcherUtils_INVERTED_COLOR as INVERTED_COLOR,
    jestMatcherUtils_BOLD_WEIGHT as BOLD_WEIGHT,
    jestMatcherUtils_DIM_COLOR as DIM_COLOR,
    jestMatcherUtils_MatcherHintOptions as MatcherHintOptions,
    jestMatcherUtils_matcherHint as matcherHint,
    jestMatcherUtils_stringify as stringify,
    jestMatcherUtils_printReceived as printReceived,
    jestMatcherUtils_printExpected as printExpected,
    jestMatcherUtils_DiffOptions as DiffOptions,
    jestMatcherUtils_diff as diff,
  };
}

declare type Tester = (a: any, b: any) => boolean | undefined;
declare type MatcherState = {
    assertionCalls: number;
    currentTestName?: string;
    dontThrow?: () => void;
    error?: Error;
    equals: (a: unknown, b: unknown, customTesters?: Array<Tester>, strictCheck?: boolean) => boolean;
    expand?: boolean;
    expectedAssertionsNumber?: number | null;
    expectedAssertionsNumberError?: Error;
    isExpectingAssertions?: boolean;
    isExpectingAssertionsError?: Error;
    isNot: boolean;
    promise: string;
    suppressedErrors: Array<Error>;
    testPath?: string;
    utils: typeof jestMatcherUtils & {
        iterableEquality: Tester;
        subsetEquality: Tester;
    };
};
declare type SyncExpectationResult = {
    pass: boolean;
    message: () => string;
};
declare type AsyncExpectationResult = Promise<SyncExpectationResult>;
declare type ExpectationResult = SyncExpectationResult | AsyncExpectationResult;
declare type RawMatcherFn<T extends MatcherState = MatcherState> = {
    (this: T, received: any, expected: any, options?: any): ExpectationResult;
};
declare type MatchersObject<T extends MatcherState = MatcherState> = {
    [id: string]: RawMatcherFn<T>;
};

declare type Awaitable<T> = T | PromiseLike<T>;
declare type Nullable<T> = T | null | undefined;
declare type Arrayable<T> = T | Array<T>;
declare type ArgumentsType<T> = T extends (...args: infer U) => any ? U : never;
interface ModuleCache {
    promise?: Promise<any>;
    exports?: any;
    transformResult?: TransformResult;
}
interface EnvironmentReturn {
    teardown: (global: any) => Awaitable<void>;
}
interface Environment {
    name: string;
    setup(global: any): Awaitable<EnvironmentReturn>;
}
interface UserConsoleLog {
    content: string;
    type: 'stdout' | 'stderr';
    taskId?: string;
}

declare type RunMode = 'run' | 'skip' | 'only' | 'todo';
declare type TaskState = RunMode | 'pass' | 'fail';
declare type ComputeMode = 'serial' | 'concurrent';
interface TaskBase {
    id: string;
    name: string;
    mode: RunMode;
    computeMode: ComputeMode;
    suite?: Suite;
    file?: File;
    result?: TaskResult;
}
interface TaskResult {
    state: TaskState;
    start: number;
    end?: number;
    error?: unknown;
}
declare type TaskResultPack = [id: string, result: TaskResult | undefined];
interface Suite extends TaskBase {
    type: 'suite';
    tasks: Task[];
}
interface File extends Suite {
    filepath: string;
}
interface Test extends TaskBase {
    type: 'test';
    suite: Suite;
    result?: TaskResult;
}
declare type Task = Test | Suite | File;
declare type DoneCallback = (error?: any) => void;
declare type TestFunction = (done: DoneCallback) => Awaitable<void>;
declare type TestCollectorFn = (name: string, fn: TestFunction, timeout?: number) => void;
interface ConcurrentCollector {
    (name: string, fn: TestFunction, timeout?: number): void;
    only: TestCollectorFn;
    skip: TestCollectorFn;
    todo: (name: string) => void;
}
interface OnlyCollector {
    (name: string, fn: TestFunction, timeout?: number): void;
    concurrent: TestCollectorFn;
}
interface SkipCollector {
    (name: string, fn: TestFunction, timeout?: number): void;
    concurrent: TestCollectorFn;
}
interface TodoCollector {
    (name: string): void;
    concurrent: (name: string) => void;
}
interface TestCollector {
    (name: string, fn: TestFunction, timeout?: number): void;
    concurrent: ConcurrentCollector;
    only: OnlyCollector;
    skip: SkipCollector;
    todo: TodoCollector;
}
declare type HookListener<T extends any[]> = (...args: T) => Awaitable<void>;
interface SuiteHooks {
    beforeAll: HookListener<[Suite]>[];
    afterAll: HookListener<[Suite]>[];
    beforeEach: HookListener<[Test, Suite]>[];
    afterEach: HookListener<[Test, Suite]>[];
}
interface SuiteCollector {
    readonly name: string;
    readonly mode: RunMode;
    type: 'collector';
    test: TestCollector;
    tasks: (Suite | Test | SuiteCollector)[];
    collect: (file?: File) => Promise<Suite>;
    clear: () => void;
    on: <T extends keyof SuiteHooks>(name: T, ...fn: SuiteHooks[T]) => void;
}
declare type TestFactory = (test: (name: string, fn: TestFunction) => void) => Awaitable<void>;
interface RuntimeContext {
    tasks: (SuiteCollector | Test)[];
    currentSuite: SuiteCollector | null;
}

interface Reporter {
    onStart?: (files?: string[]) => Awaitable<void>;
    onFinished?: (files?: File[]) => Awaitable<void>;
    onTaskUpdate?: (pack: TaskResultPack) => Awaitable<void>;
    onWatcherStart?: () => Awaitable<void>;
    onWatcherRerun?: (files: string[], trigger: string) => Awaitable<void>;
    onUserConsoleLog?: (log: UserConsoleLog) => Awaitable<void>;
}

declare type SnapshotData = Record<string, string>;
declare type SnapshotUpdateState = 'all' | 'new' | 'none';
declare type SnapshotStateOptions = {
    updateSnapshot: SnapshotUpdateState;
    expand?: boolean;
    snapshotFormat?: OptionsReceived;
};
declare type SnapshotMatchOptions = {
    testName: string;
    received: unknown;
    key?: string;
    inlineSnapshot?: string;
    isInline: boolean;
    error?: Error;
};
interface SnapshotResult {
    filepath: string;
    added: number;
    fileDeleted: boolean;
    matched: number;
    unchecked: number;
    uncheckedKeys: Array<string>;
    unmatched: number;
    updated: number;
}
interface UncheckedSnapshot {
    filePath: string;
    keys: Array<string>;
}
interface SnapshotSummary {
    added: number;
    didUpdate: boolean;
    failure: boolean;
    filesAdded: number;
    filesRemoved: number;
    filesRemovedList: Array<string>;
    filesUnmatched: number;
    filesUpdated: number;
    matched: number;
    total: number;
    unchecked: number;
    uncheckedKeysByFile: Array<UncheckedSnapshot>;
    unmatched: number;
    updated: number;
}

interface InlineConfig {
    /**
     * Include globs for test files
     *
     * @default ['**\/*.test.ts']
     */
    include?: string[];
    /**
     * Exclude globs for test files
     * @default ['**\/node_modules\/**']
     */
    exclude?: string[];
    /**
     * Handling for dependencies inlining or externalizing
     */
    deps?: {
        /**
         * Externalize means that Vite will bypass the package to native Node.
         *
         * Externalized dependencies will not be applied Vite's transformers and resolvers.
         * And does not support HMR on reload.
         *
         * Typically, packages under `node_modules` are externalized.
         */
        external?: (string | RegExp)[];
        /**
         * Vite will process inlined modules.
         *
         * This could be helpful to handle packages that ship `.js` in ESM format (that Node can't handle).
         */
        inline?: (string | RegExp)[];
    };
    /**
     * Register apis globally
     *
     * @default false
     */
    global?: boolean;
    /**
     * Running environment
     *
     * Supports 'node', 'jsdom', 'happy-dom'
     *
     * @default 'node'
     */
    environment?: 'node' | 'jsdom' | 'happy-dom';
    /**
     * Update snapshot files
     *
     * @default false
     */
    update?: boolean;
    /**
     * Watch mode
     *
     * @default false
     */
    watch?: boolean;
    /**
     * Project root
     */
    root?: string;
    /**
     * Custom reporter for output
     */
    reporters?: Reporter | Reporter[];
    /**
     * Enable multi-threading
     *
     * @default true
     */
    threads?: boolean;
    /**
     * Maximum number of threads
     *
     * @default available CPUs
     */
    maxThreads?: number;
    /**
     * Minimum number of threads
     *
     * @default available CPUs
     */
    minThreads?: number;
    interpretDefault?: boolean;
    /**
     * Default timeout of a test in milliseconds
     *
     * @default 5000
     */
    testTimeout?: number;
    /**
     * Default timeout of a hook in milliseconds
     *
     * @default 5000
     */
    hookTimeout?: number;
    /**
     * Silent mode
     *
     * @default false
     */
    silent?: boolean;
    /**
     * Path to setup files
     */
    setupFiles?: string | string[];
    /**
     * Open Vitest UI
     * @internal WIP
     */
    open?: boolean;
    /**
     * Listen to port and serve API
     *
     * When set to true, the default port is 55555
     *
     * @internal WIP
     * @default false
     */
    api?: boolean | number;
}
interface UserConfig extends InlineConfig {
    /**
     * Path to the config file.
     *
     * Default resolving to one of:
     * - `vitest.config.js`
     * - `vitest.config.ts`
     * - `vite.config.js`
     * - `vite.config.ts`
     */
    config?: string | undefined;
    dom?: boolean;
}
interface ResolvedConfig extends Omit<Required<UserConfig>, 'config' | 'filters'> {
    config?: string;
    filters?: string[];
    depsInline: (string | RegExp)[];
    depsExternal: (string | RegExp)[];
    snapshotOptions: SnapshotStateOptions;
}

interface WorkerContext {
    port: MessagePort;
    config: ResolvedConfig;
    files: string[];
    invalidates?: string[];
}
interface RpcMap {
    fetch: [[id: string], TransformResult | null | undefined];
    log: [[UserConsoleLog], void];
    processExit: [[code?: number], void];
    onCollected: [[files: File[]], void];
    onFinished: [[], void];
    onTaskUpdate: [[pack: TaskResultPack], void];
    onWatcherStart: [[], void];
    onWatcherRerun: [[files: string[], trigger: string], void];
    snapshotSaved: [[snapshot: SnapshotResult], void];
}
declare type RpcCall = <T extends keyof RpcMap>(method: T, ...args: RpcMap[T][0]) => Promise<RpcMap[T][1]>;
declare type RpcSend = <T extends keyof RpcMap>(method: T, ...args: RpcMap[T][0]) => void;
declare type RpcPayload<T extends keyof RpcMap = keyof RpcMap> = {
    id: string;
    method: T;
    args: RpcMap[T][0];
};

declare const suite: {
    (suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
    concurrent: {
        (suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        skip(suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        only(suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        todo(suiteName: string): SuiteCollector;
    };
    skip: {
        (suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        concurrent: (suiteName: string, factory?: TestFactory | undefined) => SuiteCollector;
    };
    only: {
        (suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        concurrent: (suiteName: string, factory?: TestFactory | undefined) => SuiteCollector;
    };
    todo: {
        (suiteName: string): SuiteCollector;
        concurrent: (suiteName: string) => SuiteCollector;
    };
};
declare const test: {
    (name: string, fn: TestFunction, timeout?: number | undefined): void;
    concurrent: {
        (name: string, fn: TestFunction, timeout?: number | undefined): void;
        skip(name: string, fn: TestFunction, timeout?: number | undefined): void;
        only(name: string, fn: TestFunction, timeout?: number | undefined): void;
        todo(name: string): void;
    };
    skip: {
        (name: string, fn: TestFunction, timeout?: number | undefined): void;
        concurrent(name: string, fn: TestFunction, timeout?: number | undefined): void;
    };
    only: {
        (name: string, fn: TestFunction, timeout?: number | undefined): void;
        concurrent(name: string, fn: TestFunction, timeout?: number | undefined): void;
    };
    todo: {
        (name: string): void;
        concurrent(name: string): void;
    };
};
declare const describe: {
    (suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
    concurrent: {
        (suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        skip(suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        only(suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        todo(suiteName: string): SuiteCollector;
    };
    skip: {
        (suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        concurrent: (suiteName: string, factory?: TestFactory | undefined) => SuiteCollector;
    };
    only: {
        (suiteName: string, factory?: TestFactory | undefined): SuiteCollector;
        concurrent: (suiteName: string, factory?: TestFactory | undefined) => SuiteCollector;
    };
    todo: {
        (suiteName: string): SuiteCollector;
        concurrent: (suiteName: string) => SuiteCollector;
    };
};
declare const it: {
    (name: string, fn: TestFunction, timeout?: number | undefined): void;
    concurrent: {
        (name: string, fn: TestFunction, timeout?: number | undefined): void;
        skip(name: string, fn: TestFunction, timeout?: number | undefined): void;
        only(name: string, fn: TestFunction, timeout?: number | undefined): void;
        todo(name: string): void;
    };
    skip: {
        (name: string, fn: TestFunction, timeout?: number | undefined): void;
        concurrent(name: string, fn: TestFunction, timeout?: number | undefined): void;
    };
    only: {
        (name: string, fn: TestFunction, timeout?: number | undefined): void;
        concurrent(name: string, fn: TestFunction, timeout?: number | undefined): void;
    };
    todo: {
        (name: string): void;
        concurrent(name: string): void;
    };
};
declare global {
    namespace NodeJS {
        interface Process {
            __vitest_worker__: {
                config: ResolvedConfig;
                rpc: RpcCall;
                send: RpcSend;
                current?: Test;
                moduleCache: Map<string, ModuleCache>;
            };
        }
    }
}

declare const beforeAll: (fn: SuiteHooks['beforeAll'][0], timeout?: number | undefined) => void;
declare const afterAll: (fn: SuiteHooks['afterAll'][0], timeout?: number | undefined) => void;
declare const beforeEach: (fn: SuiteHooks['beforeEach'][0], timeout?: number | undefined) => void;
declare const afterEach: (fn: SuiteHooks['afterEach'][0], timeout?: number | undefined) => void;

declare const vitest: {
    spyOn: typeof spyOn;
    fn: typeof spy;
};

declare module 'vite' {
    interface UserConfig {
        /**
         * Options for Vitest
         */
        test?: InlineConfig;
    }
}
declare global {
    namespace Chai {
        interface ExpectStatic {
            extend(expects: MatchersObject): void;
        }
        interface Assertion {
            toMatchSnapshot(message?: string): Assertion;
            toMatchInlineSnapshot(snapshot?: string, message?: string): Assertion;
            matchSnapshot(message?: string): Assertion;
            toEqual(expected: any): void;
            toStrictEqual(expected: any): void;
            toBe(expected: any): void;
            toMatch(expected: string | RegExp): void;
            toMatchObject(expected: any): void;
            toContain(item: any): void;
            toContainEqual(item: any): void;
            toBeTruthy(): void;
            toBeFalsy(): void;
            toBeGreaterThan(num: number): void;
            toBeGreaterThanOrEqual(num: number): void;
            toBeLessThan(num: number): void;
            toBeLessThanOrEqual(num: number): void;
            toBeNaN(): void;
            toBeUndefined(): void;
            toBeNull(): void;
            toBeDefined(): void;
            toBeInstanceOf(c: any): void;
            toBeCalledTimes(n: number): void;
            toHaveLength(l: number): void;
            toHaveProperty(p: string, value?: any): void;
            toBeCloseTo(number: number, numDigits?: number): void;
            toHaveBeenCalledTimes(n: number): void;
            toHaveBeenCalledOnce(): void;
            toHaveBeenCalled(): void;
            toBeCalled(): void;
            toHaveBeenCalledWith(...args: any[]): void;
            toBeCalledWith(...args: any[]): void;
            toHaveBeenNthCalledWith(n: number, ...args: any[]): void;
            nthCalledWith(n: number, ...args: any[]): void;
            toHaveBeenLastCalledWith(...args: any[]): void;
            lastCalledWith(...args: any[]): void;
            toThrow(expected?: string | RegExp): void;
            toThrowError(expected?: string | RegExp): void;
            toReturn(): void;
            toHaveReturned(): void;
            toReturnTimes(times: number): void;
            toHaveReturnedTimes(times: number): void;
            toReturnWith(value: any): void;
            toHaveReturnedWith(value: any): void;
            toHaveLastReturnedWith(value: any): void;
            lastReturnedWith(value: any): void;
            toHaveNthReturnedWith(nthCall: number, value: any): void;
            nthReturnedWith(nthCall: number, value: any): void;
        }
    }
}

export { ArgumentsType, Arrayable, Awaitable, ComputeMode, DoneCallback, Environment, EnvironmentReturn, File, HookListener, InlineConfig, ModuleCache, Nullable, Reporter, ResolvedConfig, RpcCall, RpcMap, RpcPayload, RpcSend, RunMode, RuntimeContext, SnapshotData, SnapshotMatchOptions, SnapshotResult, SnapshotStateOptions, SnapshotSummary, SnapshotUpdateState, Suite, SuiteCollector, SuiteHooks, Task, TaskBase, TaskResult, TaskResultPack, TaskState, Test, TestCollector, TestFactory, TestFunction, UncheckedSnapshot, UserConfig, UserConsoleLog, WorkerContext, afterAll, afterEach, beforeAll, beforeEach, describe, it, suite, test, vitest };
