import { resolve, dirname } from 'path';
import { n as nanoid } from './index-9e71c815.js';
import { c as distDir } from './constants-adef7ffb.js';
import { builtinModules, createRequire } from 'module';
import { pathToFileURL, fileURLToPath } from 'url';
import vm from 'vm';
import { s as slash } from './utils-9dcc4050.js';
import { s as send } from './rpc-7de86f29.js';

const defaultInline = [
  "vitest/dist",
  "@vue",
  "@vueuse",
  "vue-demi",
  "vue",
  /virtual:/,
  /\.ts$/,
  /\/esm\/.*\.js$/,
  /\.(es|esm|esm-browser|esm-bundler|es6).js$/
];
const depsExternal = [
  /\.cjs.js$/
];
const isWindows = process.platform === "win32";
const stubRequests = {
  "/@vite/client": {
    injectQuery: (id) => id,
    createHotContext() {
      return {
        accept: () => {
        },
        prune: () => {
        }
      };
    },
    updateStyle() {
    }
  }
};
async function interpretedImport(path, interpretDefault) {
  const mod = await import(path);
  if (interpretDefault && "__esModule" in mod && "default" in mod) {
    const defaultExport = mod.default;
    if (!("default" in defaultExport)) {
      Object.defineProperty(defaultExport, "default", {
        enumerable: true,
        configurable: true,
        get() {
          return defaultExport;
        }
      });
    }
    return defaultExport;
  }
  return mod;
}
let SOURCEMAPPING_URL = "sourceMa";
SOURCEMAPPING_URL += "ppingURL";
async function withInlineSourcemap(result) {
  const { code, map } = result;
  if (code.includes(`${SOURCEMAPPING_URL}=`))
    return result;
  if (map)
    result.code = `${code}

//# ${SOURCEMAPPING_URL}=data:application/json;charset=utf-8;base64,${Buffer.from(JSON.stringify(map), "utf-8").toString("base64")}`;
  return result;
}
async function executeInViteNode(options) {
  const { moduleCache, root, files, fetch } = options;
  const externaled = new Set(builtinModules);
  const result = [];
  for (const file of files)
    result.push(await cachedRequest(`/@fs/${slash(resolve(file))}`, []));
  return result;
  async function directRequest(id, fsPath, callstack) {
    callstack = [...callstack, id];
    const request = async (dep) => {
      var _a;
      if (callstack.includes(dep)) {
        const cacheKey = toFilePath(dep, root);
        if (!((_a = moduleCache.get(cacheKey)) == null ? void 0 : _a.exports))
          throw new Error(`Circular dependency detected
Stack:
${[...callstack, dep].reverse().map((p) => `- ${p}`).join("\n")}`);
        return moduleCache.get(cacheKey).exports;
      }
      return cachedRequest(dep, callstack);
    };
    if (id in stubRequests)
      return stubRequests[id];
    const result2 = await fetch(id);
    if (!result2)
      throw new Error(`failed to load ${id}`);
    if (process.env.NODE_V8_COVERAGE)
      withInlineSourcemap(result2);
    const url = pathToFileURL(fsPath).href;
    const exports = {};
    setCache(fsPath, { transformResult: result2, exports });
    const __filename = fileURLToPath(url);
    const context = {
      require: createRequire(url),
      exports,
      __filename,
      __dirname: dirname(__filename),
      __vite_ssr_import__: request,
      __vite_ssr_dynamic_import__: request,
      __vite_ssr_exports__: exports,
      __vite_ssr_exportAll__: (obj) => exportAll(exports, obj),
      __vite_ssr_import_meta__: { url }
    };
    const fn = vm.runInThisContext(`async (${Object.keys(context).join(",")})=>{${result2.code}
}`, {
      filename: fsPath,
      lineOffset: 0
    });
    await fn(...Object.values(context));
    return exports;
  }
  function setCache(id, mod) {
    if (!moduleCache.has(id))
      moduleCache.set(id, mod);
    else
      Object.assign(moduleCache.get(id), mod);
  }
  async function cachedRequest(rawId, callstack) {
    var _a, _b;
    const id = normalizeId(rawId);
    if (externaled.has(id))
      return interpretedImport(id, options.interpretDefault);
    const fsPath = toFilePath(id, root);
    const importPath = patchWindowsImportPath(fsPath);
    if (externaled.has(importPath) || await shouldExternalize(importPath, options)) {
      externaled.add(importPath);
      return interpretedImport(importPath, options.interpretDefault);
    }
    if ((_a = moduleCache.get(fsPath)) == null ? void 0 : _a.promise)
      return (_b = moduleCache.get(fsPath)) == null ? void 0 : _b.promise;
    const promise = directRequest(id, fsPath, callstack);
    setCache(fsPath, { promise });
    return await promise;
  }
  function exportAll(exports, sourceModule) {
    for (const key in sourceModule) {
      if (key !== "default") {
        try {
          Object.defineProperty(exports, key, {
            enumerable: true,
            configurable: true,
            get() {
              return sourceModule[key];
            }
          });
        } catch (_err) {
        }
      }
    }
  }
}
function normalizeId(id) {
  if (id && id.startsWith("/@id/__x00__"))
    id = `\0${id.slice("/@id/__x00__".length)}`;
  if (id && id.startsWith("/@id/"))
    id = id.slice("/@id/".length);
  if (id.startsWith("__vite-browser-external:"))
    id = id.slice("__vite-browser-external:".length);
  if (id.startsWith("node:"))
    id = id.slice("node:".length);
  return id;
}
async function shouldExternalize(id, config) {
  if (matchExternalizePattern(id, config.inline))
    return false;
  if (matchExternalizePattern(id, config.external))
    return true;
  if (matchExternalizePattern(id, depsExternal))
    return true;
  if (matchExternalizePattern(id, defaultInline))
    return false;
  return id.includes("/node_modules/");
}
function toFilePath(id, root) {
  id = slash(id);
  let absolute = id.startsWith("/@fs/") ? id.slice(4) : id.startsWith(dirname(root)) ? id : id.startsWith("/") ? slash(resolve(root, id.slice(1))) : id;
  if (absolute.startsWith("//"))
    absolute = absolute.slice(1);
  return isWindows && absolute.startsWith("/") ? pathToFileURL(absolute.slice(1)).href : absolute;
}
function matchExternalizePattern(id, patterns) {
  for (const ex of patterns) {
    if (typeof ex === "string") {
      if (id.includes(`/node_modules/${ex}/`))
        return true;
    } else {
      if (ex.test(id))
        return true;
    }
  }
  return false;
}
function patchWindowsImportPath(path) {
  if (path.match(/^\w:\//))
    return `/${path}`;
  else
    return path;
}

let _run;
const moduleCache = new Map();
async function init(ctx) {
  if (_run)
    return _run;
  const processExit = process.exit;
  process.on("beforeExit", (code) => {
    send("processExit", code);
  });
  process.exit = (code = process.exitCode || 0) => {
    send("processExit", code);
    return processExit(code);
  };
  const { config } = ctx;
  _run = (await executeInViteNode({
    root: config.root,
    files: [
      resolve(distDir, "entry.js")
    ],
    fetch(id) {
      return process.__vitest_worker__.rpc("fetch", id);
    },
    inline: config.depsInline,
    external: config.depsExternal,
    interpretDefault: config.interpretDefault,
    moduleCache
  }))[0].run;
  return _run;
}
async function run(ctx) {
  process.stdout.write("\0");
  const { config, port } = ctx;
  const rpcPromiseMap = new Map();
  process.__vitest_worker__ = {
    moduleCache,
    config,
    rpc: (method, ...args) => {
      return new Promise((resolve2, reject) => {
        const id = nanoid();
        rpcPromiseMap.set(id, { resolve: resolve2, reject });
        port.postMessage({ method, args, id });
      });
    },
    send(method, ...args) {
      port.postMessage({ method, args });
    }
  };
  port.addListener("message", async (data) => {
    const api = rpcPromiseMap.get(data.id);
    if (api) {
      if (data.error)
        api.reject(data.error);
      else
        api.resolve(data.result);
    }
  });
  const run2 = await init(ctx);
  if (ctx.invalidates)
    ctx.invalidates.forEach((i) => moduleCache.delete(i));
  ctx.files.forEach((i) => moduleCache.delete(i));
  return run2(ctx.files, ctx.config);
}

export { run as default, init };
